# coding: utf-8

## SSMotionControl.py
#  for SSMotionControlDotNet.dll

# clr Use clr import pythonnet
import clr

# namespace
clr.AddReference('SSMotionControlDotNet')

# Import on the .NET
from SurugaSeiki import MotionControl
from SurugaSeiki import McError
from SurugaSeiki import CurOnOff
from SurugaSeiki import StageType
from SurugaSeiki import UnitType

# System import
import System

class SSMotionControl:
    """SSMotionControl control class"""
    POS_NC = float(MotionControl.PositionNotControl.ToString())

    CUR_ON = 0
    CUR_OFF = 1
    CUR_NC = 2

    STAGE_NORMAL = 0
    STAGE_SINE_MOTION = 1
    STAGE_ROTARY = 2

    UNIT_PULSE = 0
    UNIT_MICROMETER = 1
    UNIT_MILLIMETER = 2
    UNIT_DEGREE = 3
    UNIT_MILLIRADIAN = 4

    __AXIS_MAX = 8
    __CTRL_MAX = 2
    __SPEED_TABLE_MAX = 16

    __mc = MotionControl()
    
    def Open(self):
        """Connect Motion controllers (If one controller is connected, it will take several seconds to return). 
        Returns
            err     result: true = success, false = failure
        """
        err = self.__mc.Open()
        return (err == McError.NoError)

    def Close(self):
        """Disconnect
        Returns
            err		result: true = success, false = failure
        """
        self.__mc.Close()

    def GetID(self):
        """機種ID取得
        引数
            err		result: true = success, false = failure
            id		Set ID: 0-1
            link	Link ID0-1: false=OFF, true=ON
        """
        bl = [False]
        bl = System.Array[bool](bl)
        b = False
        err, gi, l = self.__mc.GetID(bl, b)
        if err != McError.NoError:
            return False, None, None

        id = [0] * self.__CTRL_MAX
        for i in range(min(len(gi), len(id))):
            id[i] = bool(gi[i])
        link = l

        return (err == McError.NoError), id, link

    def SetID(self, id #0-1
        , link ): #false: OFF, true: ON
        """Set ID
        引数
            id		Set ID: 0-1
            link	Link ID0-1: false=OFF, true=ON
        戻り値
            err		result: true = success, false = failure
        """
        err = self.__mc.SetID(id, link)
        return (err == McError.NoError)

    def GetStatus(self):
        """GetStatus 
        Returns
            err		result: true = success, false = failure 
            busy	BUSY state of each axis: true=Moving, false=Stopped (busy[0]: axis1, ... , busy[7]: axis8, busy.Length: Count of axes)
            mlCCW	ML-CCW state of each axis: true=Detected, false=Not detected (mlCCW[0]: axis1, ... , mlCCW[7]: axis8, mlCCW.Length: Count of axes)
            slCCW	SL-CCW state of each axis: true=Detected, false=Not detected (slCCW[0]: axis1, ... , slCCW[7]: axis8, slCCW.Length: Count of axes)
            slCW	SL-CW state of each axis: true=Detected, false=Not detected (slCW[0]: axis1, ... , slCW[7]: axis8, slCW.Length: Count of axes)
            mlCW	ML-CW state of each axis: true=Detected, false=Not detected (mlCW[0]: axis1, ... , mlCW[7]: axis8, mlCW.Length: Count of axes)
            compORG	COMP-ORG state of each axis: true=Completed, false=Not completed (compORG[0]: axis1, ... , compORG[7]: axis8, compORG.Length: Count of axes)
            emergency	Emergency stop of each controllers: true=Detected, false=Not detected (emergency[0]: ID0, emergency[1]: ID1, emergency.Length: Count of controllers)
        """

        
        b = [False]
        b = System.Array[bool](b)
        err, busy, mlCCW, slCCW, slCW, mlCW, compORG, emergency = self.__mc.GetStatus(b, b, b, b, b, b, b)
        if err != McError.NoError:
            return False, None, None, None, None, None, None, None

#___    for ibusy, imlCCW, islCCW, islCW, imlCW, icompORG in zip(busy, mlCCW, slCCW, slCW, mlCW, compORG):
#___        print(ibusy, imlCCW, islCCW, islCW, imlCW, icompORG)
#___    print(emergency)
        busyA = [False] * len(busy)
        for i in range(min(len(busy), len(busyA))):
            busyA[i] = bool(busy[i])
        mlCCWA = [False] * len(mlCCW)
        for i in range(min(len(mlCCW), len(mlCCWA))):
            mlCCWA[i] = bool(mlCCW[i])
        slCCWA = [False] * len(slCCW)
        for i in range(min(len(slCCW), len(slCCWA))):
            slCCWA[i] = bool(slCCW[i])
        slCWA = [False] * len(slCW)
        for i in range(min(len(slCW), len(slCWA))):
            slCWA[i] = bool(slCW[i])
        mlCWA = [False] * len(mlCW)
        for i in range(min(len(mlCW), len(mlCWA))):
            mlCWA[i] = bool(mlCW[i])
        compORGA = [False] * len(compORG)
        for i in range(min(len(compORG), len(compORGA))):
            compORGA[i] = bool(compORG[i])
        emergencyA = [False] * len(emergency)
        for i in range(min(len(emergency), len(emergencyA))):
            emergencyA[i] = bool(emergency[i])
        return (err == McError.NoError), busyA, mlCCWA, slCCWA, slCWA, mlCWA, compORGA, emergencyA

    def GetPosition(self):
        """Get current position 
        Returns
            err		result: true = success, false = failure 
            position	Position of axes (position[0]: axis1, ... , position[7]: axis8, position.Length: Count of axes)
        """
        pos = [System.Decimal(1)]
        err, pos = self.__mc.GetPosition(System.Array[System.Decimal](pos))
        if err != McError.NoError:
            return False, None

        position = [0.0] * len(pos)
        for i in range(min(len(pos), len(position))):
            position[i] = float(System.Convert.ToDouble(pos[i]))
#___     for i in position:
#___         print(i)
#___     print(type(position), type(position]))
        return (err == McError.NoError), position

    def GetInPort(self, portNo):
        """Get Input port state 
        Parameters
            portNo	Port number: 0-15
            state	Input state: false=OFF, true=ON
        Returns
            err		result: true = success, false = failure
        """
        err, state = self.__mc.GetInPort(portNo, False)
#___     print(portNo, state)
        return (err == McError.NoError), state

    def GetInPortAll(self):
        """Get All input port state 
        Returns
            err		result: true = success, false = failure
        """
        err, word = self.__mc.GetInPortAll(0)
#___     print(word)
        return (err == McError.NoError), word

    def GO_ORG(self, *axis):
        """Origin Return 
        Parameters
            axis	Target axes: true=Drive, false=Do not drive (axis[0]: axis1, ... , axis[7]: axis8 or GO_ORG(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_ORG(axis)
#___     for i in axis:
#___         print(i)
        return (err == McError.NoError)

    def GO_HOME(self, *axis):
        """Home Position Drive 
        Parameters
            axis	Target axes: true=Drive, false=Do not drive (axis[0]: axis1, ... , axis[7]: axis8 or GO_HOME(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_HOME(axis)
        return (err == McError.NoError)

    def GO_ABS(self, *position):
        """Target Position Drive 
        Parameters
            position	Absolute position: PositionNotControl=Do not drive (position[0]: axis1, ... , position[7]: axis8 or GO_ABS(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(position), len(pos))):
            if position[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(position[i]))
#___     for i in pos:
#___         print(i)
        err = self.__mc.GO_ABS(pos)
        return (err == McError.NoError)

    def GO_CW(self, *axis):
        """Relative Position Drive (Moving direction: CW) Specify relative distance using relative distance settingSET_RELLENGTH(Decimal[]). 
        Parameters
            axis	Target axes: true=Drive, false=Do not drive (axis[0]: axis1, ... , axis[7]: axis8 or GO_CW(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_CW(axis)
        return (err == McError.NoError)

    def GO_CCW(self, *axis):
        """Relative Position Drive (Moving direction: CCW) Specify relative distance using relative distance setting SET_RELLENGTH(Decimal[]). 
        Parameters
            axis	Target axes: true=Drive, false=Do not drive (axis[0]: axis1, ... , axis[7]: axis8 or GO_CCW(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_CCW(axis)
        return (err == McError.NoError)

    def GO_LIN2(self, axisNo1, position1, axisNo2, position2):
        """2 axis linear interpolation Drive 
        Parameters
            axisNo1	Axis No.: 1-8
            position1	Absolute position of axisNo1.
            axisNo2	Axis No.: 1-8
            position2	Absolute position of axisNo2.
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_LIN2(axisNo1, System.Decimal(System.Double(position1))
            , axisNo2, System.Decimal(System.Double(position2)))
        return (err == McError.NoError)

    def GO_LIN3(self, axisNo1, position1, axisNo2, position2, axisNo3, position3):
        """3 axis linear interpolation Drive 
        Parameters
            axisNo1	Axis No.: 1-8
            position1	Absolute position of axisNo1.
            axisNo2	Axis No.: 1-8
            position2	Absolute position of axisNo2.
            axisNo3	Axis No.: 1-8
            position3	Absolute position of axisNo3.
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_LIN3(axisNo1, System.Decimal(System.Double(position1))
            , axisNo2, System.Decimal(System.Double(position2))
            , axisNo3, System.Decimal(System.Double(position3)))
        return (err == McError.NoError)

    def GO_CIRCW(self, axisNo1, arcCenter1, axisNo2, arcCenter2):
        """2 axis circular interpolation Drive (Moving direction: CW) Specify target potition using Position setting SET_TGPOS(Decimal[]). 
        Parameters
            axisNo1	Axis No.: 1-8
            arcCenter1	Center of arc on axis specified by axisNo1.
            axisNo2	Axis No.: 1-8
            arcCenter2	Center of arc on axis specified by axisNo2.
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_CIRCW(axisNo1, System.Decimal(System.Double(arcCenter1))
            , axisNo2, System.Decimal(System.Double(arcCenter2)))
        return (err == McError.NoError)

    def GO_CIRCCW(self, axisNo1, arcCenter1, axisNo2, arcCenter2):
        """2 axis circular interpolation Drive (Moving direction: CCW) Specify target potition using Position setting SET_TGPOS(Decimal[]). 
        Parameters
            axisNo1	Axis No.: 1-8
            arcCenter1	Center of arc on axis specified by axisNo1.
            axisNo2	Axis No.: 1-8
            arcCenter2	Center of arc on axis specified by axisNo2.
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_CIRCCW(axisNo1, System.Decimal(System.Double(arcCenter1))
            , axisNo2, System.Decimal(System.Double(arcCenter2)))
        return (err == McError.NoError)

    def GO_TCH(self, teachingPointNo):
        """Teaching position Drive 
        Parameters
            teachingPointNo	Teaching point No.: 0-63
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.GO_TCH(teachingPointNo)
        return (err == McError.NoError)

    def SEL_SPEED(self, *tableNo): #0-15
        """Speed setting 
        Parameters
            Speed table No.: 0-15=Table No., Other=Do not update (tableNo[0]: axis1, ... , tableNo[7]: axis8 or SEL_SPEED(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SEL_SPEED(tableNo)
        return (err == McError.NoError)

    def SET_TGPOS(self, *position):
        """Relative distance setting 
        Parameters
            position	Relative distance (Positive integer): PositionNotControl=Do not update (position[0]: axis1, ... , position[7]: axis8 or SET_RELLENGTH(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(position), len(pos))):
            if position[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(position[i]))
        err = self.__mc.SET_TGPOS(pos)
        return (err == McError.NoError)

    def SET_RELLENGTH(self, *distance):
        """Relative distance setting 
        Parameters
            distance	Relative distance (Positive integer): PositionNotControl=Do not update (position[0]: axis1, ... , position[7]: axis8 or SET_RELLENGTH(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(distance), len(pos))):
            if distance[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(distance[i]))
        err = self.__mc.SET_RELLENGTH(pos)
        return (err == McError.NoError)

    def STOP(self, *axis):
        """Apecified axis stop 
        Parameters
            axis	Target axes: true=Stop, false=Do not operation (axis[0]: axis1, ... , axis[7]: axis8 or STOP(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.STOP(axis)
        return (err == McError.NoError)

    def STOP_ALL(self):
        """All axis stop 
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.STOP_ALL()
        return (err == McError.NoError)

    def OUTP(self, portNo, state):
        """Specified output port operation 
        Parameters
            portNo	Port number: 0-15
            state	Input state: false=OFF, true=ON
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.OUTP(portNo, state)
        return (err == McError.NoError)

    def OUTP_ALL(self, word):
        """Bulk operation of all output port
        Parameters
            word	All output state (bit0: state of port No.0, ... , bit15: state of port No.15)
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.OUTP_ALL(word)
        return (err == McError.NoError)

    def SET_CURON(self, *cur):
        """Current ON/OFF settings 
        Parameters
            Current ON/OFF settings (axis[0]: axis1, ... , axis[7]: axis8 or SET_CURON(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        curA = [CurOnOff.NotControl] * self.__AXIS_MAX
        for i in range(min(len(cur), len(curA))):
            if cur[i] == self.CUR_ON:
                curA[i] = CurOnOff.CURON
            elif cur[i] == self.CUR_OFF:
                curA[i] = CurOnOff.CUROFF
        err = self.__mc.SET_CURON(curA)
        return (err == McError.NoError)

    def SetStageParameter(self, originReturnPattern #0-6
        , isLimitSensorOpen #false: Normal Closed, true: Nomal Open
        , isOriginSensorOpen #false: Normal Closed, true: Nomal Open
        , isProximityOriginSensorOpen #false: Normal Closed, true: Nomal Open
        , isEncoderEntryEnable #false: Disable, true: Enable
        , isDriveDirectionReverse #false: Forward, true: Reverse
        , stageType #Normal, SineMotion, Rotary
        , motionRadius): #0-255[mm]
        """Set stage parameters 
        Parameters
            Origin return pattern: 0-6 (originReturnPattern[0]: axis1, ... , originReturnPattern[7]: axis8)
            isLimitSensorOpen	Limit sensor: false=Normal Closed, true=Normal Open (isLimitSensorOpen[0]: axis1, ... , isLimitSensorOpen[7]: axis8)
            isOriginSensorOpen	Origin sensor: false=Normal Closed, true=Normal Open (isOriginSensorOpen[0]: axis1, ... , isOriginSensorOpen[7]: axis8)
            isProximityOriginSensorOpen	Proximity origin sensor: false=Normal Closed, true=Normal Open (isProximityOriginSensorOpen[0]: axis1, ... , isProximityOriginSensorOpen[7]: axis8)
            isEncoderEntryEnable	Encoder entry: false=Disable, true=Enable (isEncoderEntryEnable[0]: axis1, ... , isEncoderEntryEnable[7]: axis8)
            isDriveDirectionReverse	Stage drive direction: false=Forward, true=Reverse (isDriveDirectionReverse[0]: axis1, ... , isDriveDirectionReverse[7]: axis8)
            type	Type (type[0]: axis1, ... , type[7]: axis8)
            motionRadius	Motion radius: 0-255[mm] (motionRadius[0]: axis1, ... , motionRadius[7]: axis8)
        Returns
            err		result: true = success, false = failure
        """
        typeA = [self.STAGE_NORMAL] * self.__AXIS_MAX
        for i in range(min(len(stageType), len(typeA))):
            if stageType[i] == self.STAGE_NORMAL:
                typeA[i] = StageType.Normal
            elif stageType[i] == self.STAGE_SINE_MOTION:
                typeA[i] = StageType.SineMotion
            elif stageType[i] == self.STAGE_ROTARY:
                typeA[i] = StageType.Rotary
            else:
                return False
        err = self.__mc.SetStageParameter(originReturnPattern
            , isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen
            , isEncoderEntryEnable, isDriveDirectionReverse, typeA, motionRadius)
        return (err == McError.NoError)

    def SetUnit(self, unit): #pulse, um, mm, deg, mrad
        """Set unit 
        Parameters
            unit	Unit (unit[0]: axis1, ... , unit[7]: axis8 or SetUnit(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        unitA = [self.UNIT_PULSE] * self.__AXIS_MAX
        for i in range(min(len(unit), len(unitA))):
            if unit[i] == self.UNIT_PULSE:
                unitA[i] = UnitType.Pulse
            elif unit[i] == self.UNIT_MICROMETER:
                unitA[i] = UnitType.Micrometer
            elif unit[i] == self.UNIT_MILLIMETER:
                unitA[i] = UnitType.Millimeter
            elif unit[i] == self.UNIT_DEGREE:
                unitA[i] = UnitType.Degree
            elif unit[i] == self.UNIT_MILLIRADIAN:
                unitA[i] = UnitType.Milliradian
            else:
                return False
        err = self.__mc.SetUnit(unitA)
        return (err == McError.NoError)

    def SetTravelPerPulseByFullstep(self, value):
        """Set travel per pulse by fullstep 
        Parameters
            value	Travel per pulse by fullstep (value[0]: axis1, ... , value[7]: axis8 or SetTravelPerPulseByFullstep(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetTravelPerPulseByFullstep(val)
        return (err == McError.NoError)

    def SetStepAngleDirectly(value #1-500
        , isAngleDiv): #false:Disable, true:Enable
        """Set setp angle division
        Parameters
            value	setp angle:1-500
            isAngleDiv	Enable step angle setting: false:Disable, true:Enable
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetStepAngleDirectly(value, isAngleDiv)
        return (err == McError.NoError)

    def SetStartupSpeedForOriginReturn(self, pps): #1-9999[pps]
        """Set Startup Speed (Origin return) 
        Parameters
            pps	Startup speed : 1～9999pps
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetStartupSpeedForOriginReturn(pps)
        return (err == McError.NoError)

    def SetDrivingSpeedForOriginReturn(self, pps): #1-999999[pps]
        """Set Driving Speed (Origin return) 
        Parameters
            pps	Driving speed : 1～999999pps
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetDrivingSpeedForOriginReturn(pps)
        return (err == McError.NoError)

    def SetAccelerationTimeForOriginReturn(self, msec): #1-9999[msec]
        """Set Acceleration Time (Origin return) 
        Parameters
            msec	Acceleration time: 1-9999[msec]

        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetAccelerationTimeForOriginReturn(msec)
        return (err == McError.NoError)

    def SetSCurveRateForOriginReturn(self, rate): #0-100[%]
        """Set S-curve Rate (Origin return) 
        Parameters
            rate	S-curve rate: 0-100[%]
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetSCurveRateForOriginReturn(rate)
        return (err == McError.NoError)

    def SetEncoderParameter(self, multiplier #1, 2, 4
        , isCountDirectionInvert #false: Standard, true: Invert
        , isZPhaseActiveLow #false: Active High, true: Active Low
        , numberOfPositioningAdjustment): #0-15
        """Set encoder settings 
        Parameters
            multiplier	1, 2 or 4 (multiplier[0]: axis1, ... , multiplier[7]: axis8)
            isCountDirectionInvert	false=Standard , true=Invert (isCountDirectionInvert[0]: axis1, ... , isCountDirectionInvert[7]: axis8)
            isZPhaseActiveLow	false=Active high, true=Active low (isZPhaseActiveLow[0]: axis1, ... , isZPhaseActiveLow[7]: axis8)
            numberOfPositioningAdjustment	Number of positioning adjustment (Number of corrections when there is a difference between target position and encoder value.): 0-15 (numberOfPositioningAdjustment[0]: axis1, ... , numberOfPositioningAdjustment[7]: axis8)
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetEncoderParameter(multiplier
            , isCountDirectionInvert, isZPhaseActiveLow, numberOfPositioningAdjustment)
        return (err == McError.NoError)

    def SetEncoderResolutionNumerator(self, value): #1-100000
        """Set encoder resolution (Numerator) 
        Parameters
            value	Resolution (Numerator): 1-100000 (value[0]: axis1, ... , value[7]: axis8 or SetEncoderResolutionNumerator(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetEncoderResolutionNumerator(value)
        return (err == McError.NoError)

    def SetEncoderResolutionDenominator(self, value): #1-100000
        """Set encoder resolution (Denominator) 
        Parameters
            value	Resolution (Denominator): 1-100000 (value[0]: axis1, ... , value[7]: axis8 or SetEncoderResolutionDenominator(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetEncoderResolutionDenominator(value)
        return (err == McError.NoError)

    def SetSoftwareLimit(self, enabled):
        """Set software limit enabled 
        Parameters
            enabled	Software limit: false=Disabled, true=Enabled (enabled[0]: axis1, ... , enabled[7]: axis8 or SetSoftwareLimit(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetSoftwareLimit(enabled)
        return (err == McError.NoError)

    def SetCWLS(self, value):
        """Set CWLS
        Parameters
            value	CWLS (value[0]: axis1, ... , value[7]: axis8 or SetCWLS(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetCWLS(val)
        return (err == McError.NoError)

    def SetCCWLS(self, value):
        """Set CCWLS
        Parameters
            value	CCWLS (value[0]: axis1, ... , value[7]: axis8 or SetCCWLS(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetCCWLS(val)
        return (err == McError.NoError)

    def SetHomePosition(self, value):
        """Set home position 
        Parameters
            value	Home position (value[0]: axis1, ... , value[7]: axis8 or SetHomePosition(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetHomePosition(val)
        return (err == McError.NoError)

    def SetStageID(self, id):
        """Set stage ID 
        Parameters
            id	Stage ID: 0-1023 (id[0]: axis1, ... , id[7]: axis8 or SetStageID(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetStageID(id)
        return (err == McError.NoError)

    def GetStageParameter(self):
        """Get stage parameters 
        Returns
            err		result: true = success, false = failure
            Origin return pattern: 0-6 (originReturnPattern[0]: axis1, ... , originReturnPattern[7]: axis8)
            isLimitSensorOpen	Limit sensor: false=Normal Closed, true=Normal Open (isLimitSensorOpen[0]: axis1, ... , isLimitSensorOpen[7]: axis8)
            isOriginSensorOpen	Origin sensor: false=Normal Closed, true=Normal Open (isOriginSensorOpen[0]: axis1, ... , isOriginSensorOpen[7]: axis8)
            isProximityOriginSensorOpen	Proximity origin sensor: false=Normal Closed, true=Normal Open (isProximityOriginSensorOpen[0]: axis1, ... , isProximityOriginSensorOpen[7]: axis8)
            isEncoderEntryEnable	Encoder entry: false=Disable, true=Enable (isEncoderEntryEnable[0]: axis1, ... , isEncoderEntryEnable[7]: axis8)
            isDriveDirectionReverse	Stage drive direction: false=Forward, true=Reverse (isDriveDirectionReverse[0]: axis1, ... , isDriveDirectionReverse[7]: axis8)
            type	Type (type[0]: axis1, ... , type[7]: axis8)
            motionRadius	Motion radius: 0-255[mm] (motionRadius[0]: axis1, ... , motionRadius[7]: axis8)
        """
        i = [0]
        i = System.Array[int](i)
        b = [False]
        b = System.Array[bool](b)
        t = [StageType.Normal]
        t = System.Array[StageType](t)
        err, orp, ls, os, ps, ee, dd, st, mr = self.__mc.GetStageParameter(i, b, b, b, b, b, t, i)
        if err != McError.NoError:
            return False, None, None, None, None, None, None, None, None

        originReturnPattern = [0] * self.__AXIS_MAX
        for i in range(min(len(orp), len(originReturnPattern))):
            originReturnPattern[i] = int(orp[i])
        isLimitSensorOpen = [False] * self.__AXIS_MAX
        for i in range(min(len(ls), len(isLimitSensorOpen))):
            isLimitSensorOpen[i] = bool(ls[i])
        isOriginSensorOpen = [False] * self.__AXIS_MAX
        for i in range(min(len(os), len(isOriginSensorOpen))):
            isOriginSensorOpen[i] = bool(os[i])
        isProximityOriginSensorOpen = [False] * self.__AXIS_MAX
        for i in range(min(len(ps), len(isProximityOriginSensorOpen))):
            isProximityOriginSensorOpen[i] = bool(ps[i])
        isEncoderEntryEnable = [False] * self.__AXIS_MAX
        for i in range(min(len(ee), len(isEncoderEntryEnable))):
            isEncoderEntryEnable[i] = bool(ee[i])
        isDriveDirectionReverse = [False] * self.__AXIS_MAX
        for i in range(min(len(dd), len(isDriveDirectionReverse))):
            isDriveDirectionReverse[i] = bool(dd[i])
        stageType = [self.STAGE_NORMAL] * self.__AXIS_MAX
        for i in range(min(len(st), len(stageType))):
            if (st[i] == StageType.Normal):
                stageType[i] = self.STAGE_NORMAL
            elif (st[i] == StageType.SineMotion):
                stageType[i] = self.STAGE_SINE_MOTION
            elif (st[i] == StageType.Rotary):
                stageType[i] = self.STAGE_ROTARY
        motionRadius = [0] * self.__AXIS_MAX
        for i in range(min(len(mr), len(motionRadius))):
            motionRadius[i] = int(mr[i])
#___     print(type(originReturnPattern), type(originReturnPattern[0]))
#___     print(type(isLimitSensorOpen), type(isLimitSensorOpen[0]))
#___     print(type(stageType), type(stageType[0]))
#___     for iorp, ils, ios, ips, iee, idd, ist, imr in zip(originReturnPattern
#___         , isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen
#___         , isEncoderEntryEnable, isDriveDirectionReverse, stageType, motionRadius):
#___         print(iorp, ils, ios, ips, iee, idd, ist, imr)
        return (err == McError.NoError), originReturnPattern, isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen, isEncoderEntryEnable, isDriveDirectionReverse, stageType, motionRadius

    def GetUnit(self):
        """Get unit 
        Returns
            err		result: true = success, false = failure
            unit	Unit (unit[0]: axis1, ... , unit[7]: axis8)
        """
        u = [UnitType.Pulse]
        u = System.Array[UnitType](u)
        err, un = self.__mc.GetUnit(u)
        if err != McError.NoError:
            return False, None

        unit = [self.UNIT_PULSE] * self.__AXIS_MAX
        for i in range(min(len(un), len(unit))):
            if (un[i] == UnitType.Pulse):
                unit[i] = self.UNIT_PULSE
            elif (un[i] == UnitType.Micrometer):
                unit[i] = self.UNIT_MICROMETER
            elif (un[i] == UnitType.Millimeter):
                unit[i] = self.UNIT_MILLIMETER
            elif (un[i] == UnitType.Degree):
                unit[i] = self.UNIT_DEGREE
            elif (un[i] == UnitType.Milliradian):
                unit[i] = self.UNIT_MILLIRADIAN
#___     print(type(unit), type(unit[0]))
#___     for i in unit:
#___         print(i)
        return (err == McError.NoError), unit

    def GetTravelPerPulseByFullstep(self):
        """Get travel per pulse by fullstep 
        Returns
            err		result: true = success, false = failure
            value	Travel per pulse by fullstep (value[0]: axis1, ... , value[7]: axis8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetTravelPerPulseByFullstep(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetStepAngleDirectly(self):
        """Get setp angle division
        Returns
            err		result: true = success, false = failure
            value	setp angle:1-500
            isAngleDiv	Enable step angle setting: false:Disable, true:Enable
        """
        i = [0]
        i = System.Array[int](i)
        b = [False]
        b = System.Array[bool](b)
        err, val, div = self.__mc.GetStepAngleDirectly(i, b)
        if err != McError.NoError:
            return False, None

        value = [0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = int(val[i])
        isAngleDiv = [0] * self.__AXIS_MAX
        for i in range(min(len(div), len(isAngleDiv))):
            isAngleDiv[i] = bool(div[i])
#___     print(type(value), type(value[0]))
#___     for ival, idd in zip(value):
#___         print(ival)
        return (err == McError.NoError), value, isAngleDiv

    def GetStartupSpeedForOriginReturn(self):
        """Get Startup Speed (Origin return) 
        Returns
            err		result: true = success, false = failure
            pps	Startup speed : 1～9999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetStartupSpeedForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__AXIS_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetDrivingSpeedForOriginReturn(self):
        """Get Driving Speed (Origin return) 
        Returns
            err		result: true = success, false = failure
            pps		Driving speed : 1～999999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetDrivingSpeedForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__AXIS_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetAccelerationTimeForOriginReturn(self):
        """Get Acceleration Time (Speed table) (It takes a few seconds to return.) 
        Returns
            err		result: true = success, false = failure
            msec	Acceleration time: 1-9999[msec]
        """
        i = [0]
        i = System.Array[int](i)
        err, ms = self.__mc.GetAccelerationTimeForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        msec = [0] * self.__AXIS_MAX
        for i in range(min(len(ms), len(msec))):
            msec[i] = int(ms[i])
#___     print(type(msec), type(msec[0]))
#___     for i in msec:
#___         print(i)
        return (err == McError.NoError), msec

    def GetSCurveRateForOriginReturn(self):
        """Get S-curve Rate (Origin return) 
        Returns
            err		result: true = success, false = failure
            rate	S-curve rate: 0-100[%]
        """
        i = [0]
        i = System.Array[int](i)
        err, rt = self.__mc.GetSCurveRateForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        rate = [0] * self.__AXIS_MAX
        for i in range(min(len(rt), len(rate))):
            rate[i] = int(rt[i])
#___     print(type(rate), type(rate[0]))
#___     for i in rate:
#___         print(i)
        return (err == McError.NoError), rate

    def GetEncoderParameter(self):
        """Get encoder settings 
        Returns
            err		result: true = success, false = failure
            multiplier	1, 2 or 4 (multiplier[0]: axis1, ... , multiplier[7]: axis8)
            isCountDirectionInvert	false=Standard , true=Invert (isCountDirectionInvert[0]: axis1, ... , isCountDirectionInvert[7]: axis8)
            isZPhaseActiveLow	false=Active high, true=Active low (isZPhaseActiveLow[0]: axis1, ... , isZPhaseActiveLow[7]: axis8)
            numberOfPositioningAdjustment	Number of positioning adjustment (Number of corrections when there is a difference between target position and encoder value.): 0-15 (numberOfPositioningAdjustment[0]: axis1, ... , numberOfPositioningAdjustment[7]: axis8)
        """
        i = [0]
        i = System.Array[int](i)
        b = [False]
        b = System.Array[bool](b)
        err, mp, cd, zp, pa = self.__mc.GetEncoderParameter(i, b, b, i)
        if err != McError.NoError:
            return False, None, None, None, None

        multiplier = [0] * self.__AXIS_MAX
        for i in range(min(len(mp), len(multiplier))):
            multiplier[i] = int(mp[i])
        isCountDirectionInvert = [False] * self.__AXIS_MAX
        for i in range(min(len(cd), len(isCountDirectionInvert))):
            isCountDirectionInvert[i] = bool(cd[i])
        isZPhaseActiveLow = [False] * self.__AXIS_MAX
        for i in range(min(len(zp), len(isZPhaseActiveLow))):
            isZPhaseActiveLow[i] = bool(zp[i])
        numberOfPositioningAdjustment = [0] * self.__AXIS_MAX
        for i in range(min(len(pa), len(numberOfPositioningAdjustment))):
            numberOfPositioningAdjustment[i] = int(pa[i])
#___     print(type(multiplier), type(multiplier[0]))
#___     print(type(isCountDirectionInvert), type(isCountDirectionInvert[0]))
#___     for imp, icd, izp, ipa in zip(multiplier, isCountDirectionInvert
#___         , isZPhaseActiveLow, numberOfPositioningAdjustment):
#___         print(imp, icd, izp, ipa)
        return (err == McError.NoError), multiplier, isCountDirectionInvert, isZPhaseActiveLow, numberOfPositioningAdjustment

    def GetEncoderResolutionNumerator(self):
        """Get encoder resolution (Numerator) 
        Returns
            err		result: true = success, false = failure
            value	Resolution (Numerator): 1-100000 (value[0]: axis1, ... , value[7]: axis8)
        """
        i = [0]
        i = System.Array[int](i)
        err, val = self.__mc.GetEncoderResolutionNumerator(i)
        if err != McError.NoError:
            return False, None

        value = [0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = int(val[i])
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetEncoderResolutionDenominator(self):
        """Get encoder resolution (Denominator) 
        Returns
            err		result: true = success, false = failure
            value	Resolution (Denominator): 1-100000 (value[0]: axis1, ... , value[7]: axis8)

        """
        i = [0]
        i = System.Array[int](i)
        err, val = self.__mc.GetEncoderResolutionDenominator(i)
        if err != McError.NoError:
            return False, None

        value = [0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = int(val[i])
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetSoftwareLimit(self):
        """Get software limit enabled 
        Returns
            err		result: true = success, false = failure
            enabled	Software limit: false=Disabled, true=Enabled (enabled[0]: axis1, ... , enabled[7]: axis8)

        """
        b = [False]
        b = System.Array[bool](b)
        err, en = self.__mc.GetSoftwareLimit(b)
        if err != McError.NoError:
            return False, None

        enabled = [False] * self.__AXIS_MAX
        for i in range(min(len(en), len(enabled))):
            enabled[i] = bool(en[i])
#___     print(type(enabled), type(enabled[0]))
#___     for i in enabled:
#___         print(i)
        return (err == McError.NoError), enabled

    def GetCWLS(self):
        """Get CWLS 
        Returns
            err		result: true = success, false = failure
            value	CWLS (value[0]: axis1, ... , value[7]: axis8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetCWLS(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetCCWLS(self):
        """Get CCWLS 
        Returns
            err		result: true = success, false = failure
            value	CCWLS (value[0]: axis1, ... , value[7]: axis8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetCCWLS(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetHomePosition(self):
        """Get home position 
        Returns
            err		result: true = success, false = failure
            value	Home position (value[0]: axis1, ... , value[7]: axis8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetHomePosition(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetStageID(self):
        """Get stage ID 
        Returns
            err		result: true = success, false = failure
            id	Stage ID: 0-1023 (id[0]: axis1, ... , id[7]: axis8)
        """
        i = [0]
        i = System.Array[int](i)
        err, d = self.__mc.GetStageID(i)
        if err != McError.NoError:
            return False, None

        id = [0] * self.__AXIS_MAX
        for i in range(min(len(d), len(id))):
            id[i] = int(d[i])
#___     print(type(id), type(id[0]))
#___     for i in id:
#___         print(i)
        return (err == McError.NoError), id

    def SetStartupSpeed(self, pps): #1-9999[pps]
        """Set Startup Speed (Speed table) (It takes a few seconds to return.) 
        Parameters
            pps	Startup speed : 1～9999pps
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetStartupSpeed(pps)
        return (err == McError.NoError)

    def SetDrivingSpeed(self, pps): #1-999999[pps]
        """Set Driving Speed (Speed table) (It takes a few seconds to return.) 
        Parameters
            pps	Driving speed : 1～999999pps
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetDrivingSpeed(pps)
        return (err == McError.NoError)

    def SetAccelerationTime(self, msec): #1-9999[msec]
        """Set Acceleration Time (Speed table) (It takes a few seconds to return.) 
        Parameters
            msec	Acceleration time: 1-9999[msec]
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetAccelerationTime(msec)
        return (err == McError.NoError)

    def SetSCurveRate(self, rate): #0-100[%]
        """Set S-curve Rate (Speed table) (It takes a few seconds to return.) 
        Parameters
            rate	S-curve rate: 0-100[%] 
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.SetSCurveRate(rate)
        return (err == McError.NoError)

    def GetStartupSpeed(self):
        """Get Startup Speed (Speed table) (It takes a few seconds to return.) 
        Returns
            err		result: true = success, false = failure
            pps		Startup speed : 1～9999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetStartupSpeed(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetDrivingSpeed(self):
        """Get Driving Speed (Speed table) (It takes a few seconds to return.) 
        Returns
            err		result: true = success, false = failure
            pps		Driving speed : 1～999999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetDrivingSpeed(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetAccelerationTime(self):
        """Get Acceleration Time (Speed table) (It takes a few seconds to return.) 
        Returns
            err		result: true = success, false = failure
            msec	Acceleration time: 1-9999[msec]
        """
        i = [0]
        i = System.Array[int](i)
        err, ms = self.__mc.GetAccelerationTime(i)
        if err != McError.NoError:
            return False, None

        msec = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(ms), len(msec))):
            msec[i] = int(ms[i])
#___     print(type(msec), type(msec[0]))
#___     for i in msec:
#___         print(i)
        return (err == McError.NoError), msec

    def GetSCurveRate(self):
        """Get S-curve Rate (Speed table) (It takes a few seconds to return.) 
        Returns
            err		result: true = success, false = failure
            rate	S-curve rate: 0-100[%] %
        """
        i = [0]
        i = System.Array[int](i)
        err, rt = self.__mc.GetSCurveRate(i)
        if err != McError.NoError:
            return False, None

        rate = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(rt), len(rate))):
            rate[i] = int(rt[i])
#___     print(type(rate), type(rate[0]))
#___     for i in rate:
#___         print(i)
        return (err == McError.NoError), rate

    def SetTeachingPoint(self, no #0-63
        , *position):
        """Set teaching point 
        Parameters
            no	Teaching point No.: 0-63
            position	Teaching point: PositionNotControl=Clear teaching point (position[0]: axis1, ... , position[7]: axis8 or SetTeachingPoint(axis1, axis2, ... , axis7, axis8))
        Returns
            err		result: true = success, false = failure
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(position), len(pos))):
            if position[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(position[i]))
        err = self.__mc.SetTeachingPoint(no, pos)
        return (err == McError.NoError)

    def GetTeachingPoint(self, no): #0-63
        """Get teaching point 
        Returns
            err		result: true = success, false = failure
            no		Teaching point No.: 0-63
            position	Teaching point: PositionNotControl=Teaching point is cleared (position[0]: axis1, ... , position[7]: axis8)
        """
        pos = [System.Decimal(1)]
        err, pos = self.__mc.GetTeachingPoint(no, System.Array[System.Decimal](pos))
        if err != McError.NoError:
            return False, None

        position = [self.POS_NC] * self.__AXIS_MAX
        for i in range(min(len(pos), len(position))):
            if (pos[i] == MotionControl.PositionNotControl):
                continue
            position[i] = float(System.Convert.ToDouble(pos[i]))
#___     print(type(position), type(position[0]))
#___     for i in position:
#___         if (i == self.POS_NC):
#___             print("-")
#___         else:
#___             print(i)
        return (err == McError.NoError), position

    def WriteFlashMemory(self):
        """Write flash memory (I will return when writing is complete. It takes about 10 seconds to return.) 
        Returns
            err		result: true = success, false = failure
        """
        err = self.__mc.WriteFlashMemory()
        return (err == McError.NoError)


