# coding: utf-8

## SSMotionControl.py
#  SSMotionControlDotNet.dll との ラッパー処理

# clr インポート pythonnet を利用する
import clr

# 名前空間を指定
clr.AddReference('SSMotionControlDotNet')

# .NET 側のインポート宣言
from SurugaSeiki import MotionControl
from SurugaSeiki import McError
from SurugaSeiki import CurOnOff
from SurugaSeiki import StageType
from SurugaSeiki import UnitType

# System インポート
import System

class SSMotionControl:
    """SSMotionControl制御用クラス"""
    POS_NC = float(MotionControl.PositionNotControl.ToString())

    CUR_ON = 0
    CUR_OFF = 1
    CUR_NC = 2

    STAGE_NORMAL = 0
    STAGE_SINE_MOTION = 1
    STAGE_ROTARY = 2

    UNIT_PULSE = 0
    UNIT_MICROMETER = 1
    UNIT_MILLIMETER = 2
    UNIT_DEGREE = 3
    UNIT_MILLIRADIAN = 4

    __AXIS_MAX = 8
    __CTRL_MAX = 2
    __SPEED_TABLE_MAX = 16

    __mc = MotionControl()
    
    def Open(self):
        """コントローラに接続する(コントローラが1台のみ接続の場合、制御を返すまでに数秒かかります)
        戻り値
            err     実行結果: true=成功, false=失敗            
        """
        err = self.__mc.Open()
        return (err == McError.NoError)

    def Close(self):
        """切断する
        戻り値
            err		実行結果: true=成功, false=失敗            
        """
        self.__mc.Close()

    def GetID(self):
        """機種ID取得
        引数
            err		実行結果: true=成功, false=失敗
            id		機種ID: 0-1
            link	ID0-1連携設定: false=OFF, true=ON
        """
        bl = [False]
        bl = System.Array[bool](bl)
        b = False
        err, gi, l = self.__mc.GetID(bl, b)
        if err != McError.NoError:
            return False, None, None

        id = [0] * self.__CTRL_MAX
        for i in range(min(len(gi), len(id))):
            id[i] = bool(gi[i])
        link = l

        return (err == McError.NoError), id, link

    def SetID(self, id #0-1
        , link ): #false: OFF, true: ON
        """機種ID設定
        引数
            id		機種ID: 0-1
            link	ID0-1連携設定: false=OFF, true=ON
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetID(id, link)
        return (err == McError.NoError)

    def GetStatus(self):
        """状態取得
        戻り値
            err		実行結果: true=成功, false=失敗 
            busy	各軸の移動状態: true=移動中, false=停止中 (busy[0]:軸1, ... , busy[7]:軸8, busy.Length:軸数)
            mlCCW	各軸の機械リミット(CCW側)検出状態: true=検出, false=未検出 (mlCCW[0]:軸1, ... , mlCCW[7]:軸8, mlCCW.Length:軸数)
            slCCW	各軸のソフトリミット(CCW側)検出状態: true=検出, false=未検出 (slCCW[0]:軸1, ... , slCCW[7]:軸8, slCCW.Length:軸数)
            slCW	各軸のソフトリミット(CW側)検出状態: true=検出, false=未検出 (slCW[0]:軸1, ... , slCW[7]:軸8, slCW.Length:軸数)
            mlCW	各軸の機械リミット(CW側)検出状態: true=検出, false=未検出 (mlCW[0]:軸1, ... , mlCW[7]:軸8, mlCW.Length:軸数)
            compORG	各軸の原点復帰実施状態: true=完了, false=未実施 (compORG[0]:軸1, ... , compORG[7]:軸8, compORG.Length:軸数)
            emergency	各コントローラの非常停止状態: true=非常停止検出, false=非常停止なし (emergency[0]:ID0, emergency[1]:ID1, emergency.Length:コントローラ数)
        """
        b = [False]
        b = System.Array[bool](b)
        err, busy, mlCCW, slCCW, slCW, mlCW, compORG, emergency = self.__mc.GetStatus(b, b, b, b, b, b, b)
        if err != McError.NoError:
            return False, None, None, None, None, None, None, None

#___    for ibusy, imlCCW, islCCW, islCW, imlCW, icompORG in zip(busy, mlCCW, slCCW, slCW, mlCW, compORG):
#___        print(ibusy, imlCCW, islCCW, islCW, imlCW, icompORG)
#___    print(emergency)
        busyA = [False] * len(busy)
        for i in range(min(len(busy), len(busyA))):
            busyA[i] = bool(busy[i])
        mlCCWA = [False] * len(mlCCW)
        for i in range(min(len(mlCCW), len(mlCCWA))):
            mlCCWA[i] = bool(mlCCW[i])
        slCCWA = [False] * len(slCCW)
        for i in range(min(len(slCCW), len(slCCWA))):
            slCCWA[i] = bool(slCCW[i])
        slCWA = [False] * len(slCW)
        for i in range(min(len(slCW), len(slCWA))):
            slCWA[i] = bool(slCW[i])
        mlCWA = [False] * len(mlCW)
        for i in range(min(len(mlCW), len(mlCWA))):
            mlCWA[i] = bool(mlCW[i])
        compORGA = [False] * len(compORG)
        for i in range(min(len(compORG), len(compORGA))):
            compORGA[i] = bool(compORG[i])
        emergencyA = [False] * len(emergency)
        for i in range(min(len(emergency), len(emergencyA))):
            emergencyA[i] = bool(emergency[i])
        return (err == McError.NoError), busyA, mlCCWA, slCCWA, slCWA, mlCWA, compORGA, emergencyA

    def GetPosition(self):
        """現在位置取得
        戻り値
            err		実行結果: true=成功, false=失敗 
            position	各軸の現在位置 (position[0]:軸1, ... , position[7]:軸8, position.Length:軸数)
        """
        pos = [System.Decimal(1)]
        err, pos = self.__mc.GetPosition(System.Array[System.Decimal](pos))
        if err != McError.NoError:
            return False, None

        position = [0.0] * len(pos)
        for i in range(min(len(pos), len(position))):
            position[i] = float(System.Convert.ToDouble(pos[i]))
#___     for i in position:
#___         print(i)
#___     print(type(position), type(position]))
        return (err == McError.NoError), position

    def GetInPort(self, portNo):
        """入力ポート取得
        引数
            portNo	ポート番号 (0～15)
            state	ポート状態: false=OFF, true=ON
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err, state = self.__mc.GetInPort(portNo, False)
#___     print(portNo, state)
        return (err == McError.NoError), state

    def GetInPortAll(self):
        """全入力ポート取得
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err, word = self.__mc.GetInPortAll(0)
#___     print(word)
        return (err == McError.NoError), word

    def GO_ORG(self, *axis):
        """原点復帰
        引数
            axis	軸指定: true=駆動する, false=駆動しない (axis[0]:軸1, ... , axis[7]:軸8 または GO_ORG(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_ORG(axis)
#___     for i in axis:
#___         print(i)
        return (err == McError.NoError)

    def GO_HOME(self, *axis):
        """ホームポジション
        引数
            axis	軸指定: true=駆動する, false=駆動しない (axis[0]:軸1, ... , axis[7]:軸8 または GO_HOME(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_HOME(axis)
        return (err == McError.NoError)

    def GO_ABS(self, *position):
        """絶対位置移動
        引数
            position	絶対位置: PositionNotControl=駆動しない (position[0]:軸1, ... , position[7]:軸8 または GO_ABS(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(position), len(pos))):
            if position[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(position[i]))
#___     for i in pos:
#___         print(i)
        err = self.__mc.GO_ABS(pos)
        return (err == McError.NoError)

    def GO_CW(self, *axis):
        """相対移動(CW) 移動量は相対移動量設定SET_RELLENGTHで指定してください。
        引数
            axis	軸指定: true=駆動する, false=駆動しない (axis[0]:軸1, ... , axis[7]:軸8 または GO_CW(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_CW(axis)
        return (err == McError.NoError)

    def GO_CCW(self, *axis):
        """相対移動(CCW) 移動量は相対移動量設定SET_RELLENGTHで指定してください。
        引数
            axis	軸指定: true=駆動する, false=駆動しない (axis[0]:軸1, ... , axis[7]:軸8 または GO_CCW(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_CCW(axis)
        return (err == McError.NoError)

    def GO_LIN2(self, axisNo1, position1, axisNo2, position2):
        """2軸直線補間移動
        引数
            axisNo1	軸番号 (1～8)
            position1	axisNo1で指定した軸の移動先
            axisNo2	軸番号 (1～8)
            position2	axisNo2で指定した軸の移動先
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_LIN2(axisNo1, System.Decimal(System.Double(position1))
            , axisNo2, System.Decimal(System.Double(position2)))
        return (err == McError.NoError)

    def GO_LIN3(self, axisNo1, position1, axisNo2, position2, axisNo3, position3):
        """3軸直線補間移動
        引数
            axisNo1	軸番号 (1～8)
            position1	axisNo1で指定した軸の移動先
            axisNo2	軸番号 (1～8)
            position2	axisNo2で指定した軸の移動先
            axisNo3	軸番号 (1～8)
            position3	axisNo3で指定した軸の移動先
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_LIN3(axisNo1, System.Decimal(System.Double(position1))
            , axisNo2, System.Decimal(System.Double(position2))
            , axisNo3, System.Decimal(System.Double(position3)))
        return (err == McError.NoError)

    def GO_CIRCW(self, axisNo1, arcCenter1, axisNo2, arcCenter2):
        """2軸円弧補間移動(CW) 移動先は目標位置設定SET_TGPOSで指定してください。
        引数
            axisNo1	軸番号 (1～8)
            arcCenter1	axisNo1で指定した軸における円弧の中心座標
            axisNo2	軸番号 (1～8)
            arcCenter2	axisNo2で指定した軸における円弧の中心座標
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_CIRCW(axisNo1, System.Decimal(System.Double(arcCenter1))
            , axisNo2, System.Decimal(System.Double(arcCenter2)))
        return (err == McError.NoError)

    def GO_CIRCCW(self, axisNo1, arcCenter1, axisNo2, arcCenter2):
        """2軸円弧補間移動(CCW) 移動先は目標位置設定SET_TGPOSで指定してください。
        引数
            axisNo1	軸番号 (1～8)
            arcCenter1	axisNo1で指定した軸における円弧の中心座標
            axisNo2	軸番号 (1～8)
            arcCenter2	axisNo2で指定した軸における円弧の中心座標
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_CIRCCW(axisNo1, System.Decimal(System.Double(arcCenter1))
            , axisNo2, System.Decimal(System.Double(arcCenter2)))
        return (err == McError.NoError)

    def GO_TCH(self, teachingPointNo):
        """ティーチング位置移動
        引数
            teachingPointNo	ティーチングポイント番号 (0～63)
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.GO_TCH(teachingPointNo)
        return (err == McError.NoError)

    def SEL_SPEED(self, *tableNo): #0-15
        """速度設定
        引数
            tableNo	速度テーブル番号: 0～15=テーブル番号, その他=変更しない (tableNo[0]:軸1, ... , tableNo[7]:軸8 または SEL_SPEED(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SEL_SPEED(tableNo)
        return (err == McError.NoError)

    def SET_TGPOS(self, *position):
        """目標位置設定
        引数
            position	絶対位置: PositionNotControl=変更しない (position[0]:軸1, ... , position[7]:軸8 または SET_TGPOS(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(position), len(pos))):
            if position[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(position[i]))
        err = self.__mc.SET_TGPOS(pos)
        return (err == McError.NoError)

    def SET_RELLENGTH(self, *distance):
        """相対移動量設定
        引数
            distance	移動量: PositionNotControl=変更しない (position[0]:軸1, ... , position[7]:軸8 または SET_RELLENGTH(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(distance), len(pos))):
            if distance[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(distance[i]))
        err = self.__mc.SET_RELLENGTH(pos)
        return (err == McError.NoError)

    def STOP(self, *axis):
        """停止
        引数
            axis	軸指定: true=停止する, false=制御しない (axis[0]:軸1, ... , axis[7]:軸8 または STOP(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.STOP(axis)
        return (err == McError.NoError)

    def STOP_ALL(self):
        """全軸停止
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.STOP_ALL()
        return (err == McError.NoError)

    def OUTP(self, portNo, state):
        """出力ポート制御
        引数
            portNo	ポート番号 (0～15)
            state	ポート状態: false=OFF, true=ON
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.OUTP(portNo, state)
        return (err == McError.NoError)

    def OUTP_ALL(self, word):
        """全ポート一括操作
        引数
            word	全ポート状態 (bit0:ポート番号0の値, ... , bit15:ポート番号15の値)
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.OUTP_ALL(word)
        return (err == McError.NoError)

    def SET_CURON(self, *cur):
        """電流断設定
        引数
            cur	電流断設定 (axis[0]:軸1, ... , axis[7]:軸8 または SET_CURON(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        curA = [CurOnOff.NotControl] * self.__AXIS_MAX
        for i in range(min(len(cur), len(curA))):
            if cur[i] == self.CUR_ON:
                curA[i] = CurOnOff.CURON
            elif cur[i] == self.CUR_OFF:
                curA[i] = CurOnOff.CUROFF
        err = self.__mc.SET_CURON(curA)
        return (err == McError.NoError)

    def SetStageParameter(self, originReturnPattern #0-6
        , isLimitSensorOpen #false: Normal Closed, true: Nomal Open
        , isOriginSensorOpen #false: Normal Closed, true: Nomal Open
        , isProximityOriginSensorOpen #false: Normal Closed, true: Nomal Open
        , isEncoderEntryEnable #false: Disable, true: Enable
        , isDriveDirectionReverse #false: Forward, true: Reverse
        , stageType #Normal, SineMotion, Rotary
        , motionRadius): #0-255[mm]
        """ステージパラメータ設定
        引数
            originReturnPattern	原点復帰パターン番号: 0～6 (originReturnPattern[0]:軸1, ... , originReturnPattern[7]:軸8)
            isLimitSensorOpen	リミットセンサ入力論理: false=B接点 N.C., true=A接点 N.O. (isLimitSensorOpen[0]:軸1, ... , isLimitSensorOpen[7]:軸8)
            isOriginSensorOpen	原点センサ入力論理: false=B接点 N.C., true=A接点 N.O. (isOriginSensorOpen[0]:軸1, ... , isOriginSensorOpen[7]:軸8)
            isProximityOriginSensorOpen	近接原点入力論理: false=B接点 N.C., true=A接点 N.O. (isProximityOriginSensorOpen[0]:軸1, ... , isProximityOriginSensorOpen[7]:軸8)
            isEncoderEntryEnable	エンコーダ設定: false=エンコーダなし, true=エンコーダあり (isEncoderEntryEnable[0]:軸1, ... , isEncoderEntryEnable[7]:軸8)
            isDriveDirectionReverse	動作方向: false=順方向, true=逆方向 (isDriveDirectionReverse[0]:軸1, ... , isDriveDirectionReverse[7]:軸8)
            type	ステージ種別 (type[0]:軸1, ... , type[7]:軸8)
            motionRadius	モーション半径: 0～255mm
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        typeA = [self.STAGE_NORMAL] * self.__AXIS_MAX
        for i in range(min(len(stageType), len(typeA))):
            if stageType[i] == self.STAGE_NORMAL:
                typeA[i] = StageType.Normal
            elif stageType[i] == self.STAGE_SINE_MOTION:
                typeA[i] = StageType.SineMotion
            elif stageType[i] == self.STAGE_ROTARY:
                typeA[i] = StageType.Rotary
            else:
                return False
        err = self.__mc.SetStageParameter(originReturnPattern
            , isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen
            , isEncoderEntryEnable, isDriveDirectionReverse, typeA, motionRadius)
        return (err == McError.NoError)

    def SetUnit(self, unit): #pulse, um, mm, deg, mrad
        """単位設定
        引数
            unit	単位 (unit[0]:軸1, ... , unit[7]:軸8 または SetUnit(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        unitA = [self.UNIT_PULSE] * self.__AXIS_MAX
        for i in range(min(len(unit), len(unitA))):
            if unit[i] == self.UNIT_PULSE:
                unitA[i] = UnitType.Pulse
            elif unit[i] == self.UNIT_MICROMETER:
                unitA[i] = UnitType.Micrometer
            elif unit[i] == self.UNIT_MILLIMETER:
                unitA[i] = UnitType.Millimeter
            elif unit[i] == self.UNIT_DEGREE:
                unitA[i] = UnitType.Degree
            elif unit[i] == self.UNIT_MILLIRADIAN:
                unitA[i] = UnitType.Milliradian
            else:
                return False
        err = self.__mc.SetUnit(unitA)
        return (err == McError.NoError)

    def SetTravelPerPulseByFullstep(self, value):
        """フルステップ時の1パルス移動量設定
        引数
            value	移動量 (value[0]:軸1, ... , value[7]:軸8 または SetTravelPerPulseByFullstep(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetTravelPerPulseByFullstep(val)
        return (err == McError.NoError)

    def SetStepAngleDirectly(value #1-500
        , isAngleDiv): #false: SetStepAngleDirectly無効, true: SetStepAngleDirectly有効
        """ドライバ分割設定
        引数
            value ドライバ分割比率の分母：1〜500
            isAngleDiv false: SetStepAngleDirectly無効, true: SetStepAngleDirectly有効
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetStepAngleDirectly(value, isAngleDiv)
        return (err == McError.NoError)

    def SetStartupSpeedForOriginReturn(self, pps): #1-9999[pps]
        """起動速度(原点復帰時)設定
        引数
            pps	起動速度: 1～9999pps
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetStartupSpeedForOriginReturn(pps)
        return (err == McError.NoError)

    def SetDrivingSpeedForOriginReturn(self, pps): #1-999999[pps]
        """駆動速度(原点復帰時)設定
        引数
            pps	駆動速度: 1～999999pps
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetDrivingSpeedForOriginReturn(pps)
        return (err == McError.NoError)

    def SetAccelerationTimeForOriginReturn(self, msec): #1-9999[msec]
        """加速時間(原点復帰時)設定
        引数
            msec	加速時間: 1～9999msec

        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetAccelerationTimeForOriginReturn(msec)
        return (err == McError.NoError)

    def SetSCurveRateForOriginReturn(self, rate): #0-100[%]
        """S字レート(原点復帰時)設定
        引数
            rate	S字レート: 0～100%
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetSCurveRateForOriginReturn(rate)
        return (err == McError.NoError)

    def SetEncoderParameter(self, multiplier #1, 2, 4
        , isCountDirectionInvert #false: Standard, true: Invert
        , isZPhaseActiveLow #false: Active High, true: Active Low
        , numberOfPositioningAdjustment): #0-15
        """エンコーダ設定
        引数
            multiplier	エンコーダ倍率: 1=1逓倍, 2=2逓倍, 4=4逓倍 (multiplier[0]:軸1, ... , multiplier[7]:軸8)
            isCountDirectionInvert	エンコーダカウント方向: false=標準, true=反転 (isCountDirectionInvert[0]:軸1, ... , isCountDirectionInvert[7]:軸8)
            isZPhaseActiveLow	Z相入力論理: false=正論理, true=負論理 (isZPhaseActiveLow[0]:軸1, ... , isZPhaseActiveLow[7]:軸8)
            numberOfPositioningAdjustment	位置差分調整回数(目標位置とエンコーダ値に差があった際に補正動作を行う繰り返し回数): 0～15 (numberOfPositioningAdjustment[0]:軸1, ... , numberOfPositioningAdjustment[7]:軸8)
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetEncoderParameter(multiplier
            , isCountDirectionInvert, isZPhaseActiveLow, numberOfPositioningAdjustment)
        return (err == McError.NoError)

    def SetEncoderResolutionNumerator(self, value): #1-100000
        """エンコーダ分解能比率分子設定
        引数
            value	エンコーダ分解能比率を計算する際の分子: 1～100000 (value[0]:軸1, ... , value[7]:軸8 または SetEncoderResolutionNumerator(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetEncoderResolutionNumerator(value)
        return (err == McError.NoError)

    def SetEncoderResolutionDenominator(self, value): #1-100000
        """エンコーダ分解能比率分母設定
        引数
            value	エンコーダ分解能比率を計算する際の分母: 1～100000 (value[0]:軸1, ... , value[7]:軸8 または SetEncoderResolutionDenominator(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetEncoderResolutionDenominator(value)
        return (err == McError.NoError)

    def SetSoftwareLimit(self, enabled):
        """ソフトリミット有効/無効設定
        引数
            enabled	ソフトリミット設定: false=無効, true=有効 (enabled[0]:軸1, ... , enabled[7]:軸8 または SetSoftwareLimit(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetSoftwareLimit(enabled)
        return (err == McError.NoError)

    def SetCWLS(self, value):
        """CW側ソフトリミット設定
        引数
            value	CW側ソフトリミット (value[0]:軸1, ... , value[7]:軸8 または SetCWLS(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetCWLS(val)
        return (err == McError.NoError)

    def SetCCWLS(self, value):
        """CCW側ソフトリミット設定
        引数
            value	CCW側ソフトリミット (value[0]:軸1, ... , value[7]:軸8 または SetCCWLS(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetCCWLS(val)
        return (err == McError.NoError)

    def SetHomePosition(self, value):
        """ホームポジション設定
        引数
            value	ホームポジション (value[0]:軸1, ... , value[7]:軸8 または SetHomePosition(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        val = [0] * self.__AXIS_MAX
        for i in range(min(len(value), len(val))):
            val[i] = System.Decimal(System.Double(value[i]))
        err = self.__mc.SetHomePosition(val)
        return (err == McError.NoError)

    def SetStageID(self, id):
        """ステージ機種ID設定
        引数
            id	ステージ機種ID: 0～1023 (id[0]:軸1, ... , id[7]:軸8 または SetStageID(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetStageID(id)
        return (err == McError.NoError)

    def GetStageParameter(self):
        """ステージパラメータ取得
        戻り値
            err		実行結果: true=成功, false=失敗
            originReturnPattern	原点復帰パターン番号: 0～6 (originReturnPattern[0]:軸1, ... , originReturnPattern[7]:軸8)
            isLimitSensorOpen	リミットセンサ入力論理: false=B接点 N.C., true=A接点 N.O. (isLimitSensorOpen[0]:軸1, ... , isLimitSensorOpen[7]:軸8)
            isOriginSensorOpen	原点センサ入力論理: false=B接点 N.C., true=A接点 N.O. (isOriginSensorOpen[0]:軸1, ... , isOriginSensorOpen[7]:軸8)
            isProximityOriginSensorOpen	近接原点入力論理: false=B接点 N.C., true=A接点 N.O. (isProximityOriginSensorOpen[0]:軸1, ... , isProximityOriginSensorOpen[7]:軸8)
            isEncoderEntryEnable	エンコーダ設定: false=エンコーダなし, true=エンコーダあり (isEncoderEntryEnable[0]:軸1, ... , isEncoderEntryEnable[7]:軸8)
            isDriveDirectionReverse	動作方向: false=順方向, true=逆方向 (isDriveDirectionReverse[0]:軸1, ... , isDriveDirectionReverse[7]:軸8)
            type	ステージ種別 (type[0]:軸1, ... , type[7]:軸8)
            motionRadius	モーション半径: 0～255mm
        """
        i = [0]
        i = System.Array[int](i)
        b = [False]
        b = System.Array[bool](b)
        t = [StageType.Normal]
        t = System.Array[StageType](t)
        err, orp, ls, os, ps, ee, dd, st, mr = self.__mc.GetStageParameter(i, b, b, b, b, b, t, i)
        if err != McError.NoError:
            return False, None, None, None, None, None, None, None, None

        originReturnPattern = [0] * self.__AXIS_MAX
        for i in range(min(len(orp), len(originReturnPattern))):
            originReturnPattern[i] = int(orp[i])
        isLimitSensorOpen = [False] * self.__AXIS_MAX
        for i in range(min(len(ls), len(isLimitSensorOpen))):
            isLimitSensorOpen[i] = bool(ls[i])
        isOriginSensorOpen = [False] * self.__AXIS_MAX
        for i in range(min(len(os), len(isOriginSensorOpen))):
            isOriginSensorOpen[i] = bool(os[i])
        isProximityOriginSensorOpen = [False] * self.__AXIS_MAX
        for i in range(min(len(ps), len(isProximityOriginSensorOpen))):
            isProximityOriginSensorOpen[i] = bool(ps[i])
        isEncoderEntryEnable = [False] * self.__AXIS_MAX
        for i in range(min(len(ee), len(isEncoderEntryEnable))):
            isEncoderEntryEnable[i] = bool(ee[i])
        isDriveDirectionReverse = [False] * self.__AXIS_MAX
        for i in range(min(len(dd), len(isDriveDirectionReverse))):
            isDriveDirectionReverse[i] = bool(dd[i])
        stageType = [self.STAGE_NORMAL] * self.__AXIS_MAX
        for i in range(min(len(st), len(stageType))):
            if (st[i] == StageType.Normal):
                stageType[i] = self.STAGE_NORMAL
            elif (st[i] == StageType.SineMotion):
                stageType[i] = self.STAGE_SINE_MOTION
            elif (st[i] == StageType.Rotary):
                stageType[i] = self.STAGE_ROTARY
        motionRadius = [0] * self.__AXIS_MAX
        for i in range(min(len(mr), len(motionRadius))):
            motionRadius[i] = int(mr[i])
#___     print(type(originReturnPattern), type(originReturnPattern[0]))
#___     print(type(isLimitSensorOpen), type(isLimitSensorOpen[0]))
#___     print(type(stageType), type(stageType[0]))
#___     for iorp, ils, ios, ips, iee, idd, ist, imr in zip(originReturnPattern
#___         , isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen
#___         , isEncoderEntryEnable, isDriveDirectionReverse, stageType, motionRadius):
#___         print(iorp, ils, ios, ips, iee, idd, ist, imr)
        return (err == McError.NoError), originReturnPattern, isLimitSensorOpen, isOriginSensorOpen, isProximityOriginSensorOpen, isEncoderEntryEnable, isDriveDirectionReverse, stageType, motionRadius

    def GetUnit(self):
        """単位取得
        戻り値
            err		実行結果: true=成功, false=失敗
            unit	単位 (unit[0]:軸1, ... , unit[7]:軸8)
        """
        u = [UnitType.Pulse]
        u = System.Array[UnitType](u)
        err, un = self.__mc.GetUnit(u)
        if err != McError.NoError:
            return False, None

        unit = [self.UNIT_PULSE] * self.__AXIS_MAX
        for i in range(min(len(un), len(unit))):
            if (un[i] == UnitType.Pulse):
                unit[i] = self.UNIT_PULSE
            elif (un[i] == UnitType.Micrometer):
                unit[i] = self.UNIT_MICROMETER
            elif (un[i] == UnitType.Millimeter):
                unit[i] = self.UNIT_MILLIMETER
            elif (un[i] == UnitType.Degree):
                unit[i] = self.UNIT_DEGREE
            elif (un[i] == UnitType.Milliradian):
                unit[i] = self.UNIT_MILLIRADIAN
#___     print(type(unit), type(unit[0]))
#___     for i in unit:
#___         print(i)
        return (err == McError.NoError), unit

    def GetTravelPerPulseByFullstep(self):
        """フルステップ時の1パルス移動量取得
        戻り値
            err		実行結果: true=成功, false=失敗
            value	移動量 (value[0]:軸1, ... , value[7]:軸8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetTravelPerPulseByFullstep(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetStepAngleDirectly(self):
        """ドライバ分割数取得
        戻り値
            err		実行結果: true=成功, false=失敗
            value ドライバ分割比率の分母：1〜500
            isAngleDiv false: SetStepAngleDirectly無効, true: SetStepAngleDirectly有効
        """
        i = [0]
        i = System.Array[int](i)
        b = [False]
        b = System.Array[bool](b)
        err, val, div = self.__mc.GetStepAngleDirectly(i, b)
        if err != McError.NoError:
            return False, None

        value = [0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = int(val[i])
        isAngleDiv = [0] * self.__AXIS_MAX
        for i in range(min(len(div), len(isAngleDiv))):
            isAngleDiv[i] = bool(div[i])
#___     print(type(value), type(value[0]))
#___     for ival, idd in zip(value):
#___         print(ival)
        return (err == McError.NoError), value, isAngleDiv

    def GetStartupSpeedForOriginReturn(self):
        """起動速度(原点復帰時)取得
        戻り値
            err		実行結果: true=成功, false=失敗
            pps	起動速度: 1～9999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetStartupSpeedForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__AXIS_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetDrivingSpeedForOriginReturn(self):
        """駆動速度(原点復帰時)取得
        戻り値
            err		実行結果: true=成功, false=失敗
            pps		駆動速度: 1～999999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetDrivingSpeedForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__AXIS_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetAccelerationTimeForOriginReturn(self):
        """加速時間(原点復帰時)取得
        戻り値
            err		実行結果: true=成功, false=失敗
            msec	加速時間: 1～9999msec
        """
        i = [0]
        i = System.Array[int](i)
        err, ms = self.__mc.GetAccelerationTimeForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        msec = [0] * self.__AXIS_MAX
        for i in range(min(len(ms), len(msec))):
            msec[i] = int(ms[i])
#___     print(type(msec), type(msec[0]))
#___     for i in msec:
#___         print(i)
        return (err == McError.NoError), msec

    def GetSCurveRateForOriginReturn(self):
        """S字レート(原点復帰時)取得
        戻り値
            err		実行結果: true=成功, false=失敗
            rate	S字レート: 0～100%
        """
        i = [0]
        i = System.Array[int](i)
        err, rt = self.__mc.GetSCurveRateForOriginReturn(i)
        if err != McError.NoError:
            return False, None

        rate = [0] * self.__AXIS_MAX
        for i in range(min(len(rt), len(rate))):
            rate[i] = int(rt[i])
#___     print(type(rate), type(rate[0]))
#___     for i in rate:
#___         print(i)
        return (err == McError.NoError), rate

    def GetEncoderParameter(self):
        """エンコーダ設定取得
        戻り値
            err		実行結果: true=成功, false=失敗
            multiplier	エンコーダ倍率: 1=1逓倍, 2=2逓倍, 4=4逓倍 (multiplier[0]:軸1, ... , multiplier[7]:軸8)
            isCountDirectionInvert	エンコーダカウント方向: false=標準, true=反転 (isCountDirectionInvert[0]:軸1, ... , isCountDirectionInvert[7]:軸8)
            isZPhaseActiveLow	Z相入力論理: false=正論理, true=負論理 (isZPhaseActiveLow[0]:軸1, ... , isZPhaseActiveLow[7]:軸8)
            numberOfPositioningAdjustment	位置差分調整回数(目標位置とエンコーダ値に差があった際に補正動作を行う繰り返し回数): 0～15 (numberOfPositioningAdjustment[0]:軸1, ... , numberOfPositioningAdjustment[7]:軸8)
        """
        i = [0]
        i = System.Array[int](i)
        b = [False]
        b = System.Array[bool](b)
        err, mp, cd, zp, pa = self.__mc.GetEncoderParameter(i, b, b, i)
        if err != McError.NoError:
            return False, None, None, None, None

        multiplier = [0] * self.__AXIS_MAX
        for i in range(min(len(mp), len(multiplier))):
            multiplier[i] = int(mp[i])
        isCountDirectionInvert = [False] * self.__AXIS_MAX
        for i in range(min(len(cd), len(isCountDirectionInvert))):
            isCountDirectionInvert[i] = bool(cd[i])
        isZPhaseActiveLow = [False] * self.__AXIS_MAX
        for i in range(min(len(zp), len(isZPhaseActiveLow))):
            isZPhaseActiveLow[i] = bool(zp[i])
        numberOfPositioningAdjustment = [0] * self.__AXIS_MAX
        for i in range(min(len(pa), len(numberOfPositioningAdjustment))):
            numberOfPositioningAdjustment[i] = int(pa[i])
#___     print(type(multiplier), type(multiplier[0]))
#___     print(type(isCountDirectionInvert), type(isCountDirectionInvert[0]))
#___     for imp, icd, izp, ipa in zip(multiplier, isCountDirectionInvert
#___         , isZPhaseActiveLow, numberOfPositioningAdjustment):
#___         print(imp, icd, izp, ipa)
        return (err == McError.NoError), multiplier, isCountDirectionInvert, isZPhaseActiveLow, numberOfPositioningAdjustment

    def GetEncoderResolutionNumerator(self):
        """エンコーダ分解能比率分子取得
        戻り値
            err		実行結果: true=成功, false=失敗
            value	エンコーダ分解能比率を計算する際の分子: 1～100000 (value[0]:軸1, ... , value[7]:軸8)
        """
        i = [0]
        i = System.Array[int](i)
        err, val = self.__mc.GetEncoderResolutionNumerator(i)
        if err != McError.NoError:
            return False, None

        value = [0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = int(val[i])
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetEncoderResolutionDenominator(self):
        """エンコーダ分解能比率分母取得
        戻り値
            err		実行結果: true=成功, false=失敗
            value	エンコーダ分解能比率を計算する際の分母: 1～100000 (value[0]:軸1, ... , value[7]:軸8)

        """
        i = [0]
        i = System.Array[int](i)
        err, val = self.__mc.GetEncoderResolutionDenominator(i)
        if err != McError.NoError:
            return False, None

        value = [0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = int(val[i])
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetSoftwareLimit(self):
        """ソフトリミット有効/無効取得
        戻り値
            err		実行結果: true=成功, false=失敗
            enabled	ソフトリミット設定: false=無効, true=有効 (enabled[0]:軸1, ... , enabled[7]:軸8)

        """
        b = [False]
        b = System.Array[bool](b)
        err, en = self.__mc.GetSoftwareLimit(b)
        if err != McError.NoError:
            return False, None

        enabled = [False] * self.__AXIS_MAX
        for i in range(min(len(en), len(enabled))):
            enabled[i] = bool(en[i])
#___     print(type(enabled), type(enabled[0]))
#___     for i in enabled:
#___         print(i)
        return (err == McError.NoError), enabled

    def GetCWLS(self):
        """CW側ソフトリミット取得
        戻り値
            err		実行結果: true=成功, false=失敗
            value	CW側ソフトリミット (value[0]:軸1, ... , value[7]:軸8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetCWLS(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetCCWLS(self):
        """CCW側ソフトリミット設定取得
        戻り値
            err		実行結果: true=成功, false=失敗
            value	CCW側ソフトリミット (value[0]:軸1, ... , value[7]:軸8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetCCWLS(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetHomePosition(self):
        """ホームポジション取得
        戻り値
            err		実行結果: true=成功, false=失敗
            value	ホームポジション (value[0]:軸1, ... , value[7]:軸8)
        """
        val = [System.Decimal(1)]
        err, val = self.__mc.GetHomePosition(System.Array[System.Decimal](val))
        if err != McError.NoError:
            return False, None

        value = [0.0] * self.__AXIS_MAX
        for i in range(min(len(val), len(value))):
            value[i] = float(System.Convert.ToDouble(val[i]))
#___     print(type(value), type(value[0]))
#___     for i in value:
#___         print(i)
        return (err == McError.NoError), value

    def GetStageID(self):
        """ステージ機種ID取得
        戻り値
            err		実行結果: true=成功, false=失敗
            id	ステージ機種ID: 0～1023 (id[0]:軸1, ... , id[7]:軸8)
        """
        i = [0]
        i = System.Array[int](i)
        err, d = self.__mc.GetStageID(i)
        if err != McError.NoError:
            return False, None

        id = [0] * self.__AXIS_MAX
        for i in range(min(len(d), len(id))):
            id[i] = int(d[i])
#___     print(type(id), type(id[0]))
#___     for i in id:
#___         print(i)
        return (err == McError.NoError), id

    def SetStartupSpeed(self, pps): #1-9999[pps]
        """起動速度(速度テーブル)設定 (制御を返すまでに数秒かかります。)
        引数
            pps	起動速度: 1～9999pps
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetStartupSpeed(pps)
        return (err == McError.NoError)

    def SetDrivingSpeed(self, pps): #1-999999[pps]
        """駆動速度(速度テーブル)設定 (制御を返すまでに数秒かかります。)
        引数
            pps	駆動速度: 1～999999pps
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetDrivingSpeed(pps)
        return (err == McError.NoError)

    def SetAccelerationTime(self, msec): #1-9999[msec]
        """加速時間(速度テーブル)設定 (制御を返すまでに数秒かかります。)
        引数
            msec	加速時間: 1～9999msec
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetAccelerationTime(msec)
        return (err == McError.NoError)

    def SetSCurveRate(self, rate): #0-100[%]
        """S字レート(速度テーブル)設定 (制御を返すまでに数秒かかります。)
        引数
            rate	S字レート: 1～100
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.SetSCurveRate(rate)
        return (err == McError.NoError)

    def GetStartupSpeed(self):
        """起動速度(速度テーブル)取得 (制御を返すまでに数秒かかります。)
        戻り値
            err		実行結果: true=成功, false=失敗
            pps		起動速度: 1～9999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetStartupSpeed(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetDrivingSpeed(self):
        """駆動速度(速度テーブル)取得 (制御を返すまでに数秒かかります。)
        戻り値
            err		実行結果: true=成功, false=失敗
            pps		駆動速度: 1～999999pps
        """
        i = [0]
        i = System.Array[int](i)
        err, p = self.__mc.GetDrivingSpeed(i)
        if err != McError.NoError:
            return False, None

        pps = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(p), len(pps))):
            pps[i] = int(p[i])
#___     print(type(pps), type(pps[0]))
#___     for i in pps:
#___         print(i)
        return (err == McError.NoError), pps

    def GetAccelerationTime(self):
        """加速時間(速度テーブル)取得 (制御を返すまでに数秒かかります。)
        戻り値
            err		実行結果: true=成功, false=失敗
            msec	加速時間: 1～9999msec
        """
        i = [0]
        i = System.Array[int](i)
        err, ms = self.__mc.GetAccelerationTime(i)
        if err != McError.NoError:
            return False, None

        msec = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(ms), len(msec))):
            msec[i] = int(ms[i])
#___     print(type(msec), type(msec[0]))
#___     for i in msec:
#___         print(i)
        return (err == McError.NoError), msec

    def GetSCurveRate(self):
        """S字レート(速度テーブル)取得 (制御を返すまでに数秒かかります。)
        戻り値
            err		実行結果: true=成功, false=失敗
            rate	S字レート: 1～100%
        """
        i = [0]
        i = System.Array[int](i)
        err, rt = self.__mc.GetSCurveRate(i)
        if err != McError.NoError:
            return False, None

        rate = [0] * self.__SPEED_TABLE_MAX
        for i in range(min(len(rt), len(rate))):
            rate[i] = int(rt[i])
#___     print(type(rate), type(rate[0]))
#___     for i in rate:
#___         print(i)
        return (err == McError.NoError), rate

    def SetTeachingPoint(self, no #0-63
        , *position):
        """ティーチングポイント設定
        引数
            no	ティーチングポイント番号: 0～63
            position	ティーチングポイント: PositionNotControl=クリアする (position[0]:軸1, ... , position[7]:軸8 または SetTeachingPoint(軸1, 軸2, ... , 軸7, 軸8))
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        pos = [MotionControl.PositionNotControl] * self.__AXIS_MAX
        for i in range(min(len(position), len(pos))):
            if position[i] == self.POS_NC:
                continue
            pos[i] = System.Decimal(System.Double(position[i]))
        err = self.__mc.SetTeachingPoint(no, pos)
        return (err == McError.NoError)

    def GetTeachingPoint(self, no): #0-63
        """ティーチングポイント取得
        戻り値
            err		実行結果: true=成功, false=失敗
            no		ティーチングポイント番号: 0～63
            position	ティーチングポイント: PositionNotControl=クリアされている (position[0]:軸1, ... , position[7]:軸8)
        """
        pos = [System.Decimal(1)]
        err, pos = self.__mc.GetTeachingPoint(no, System.Array[System.Decimal](pos))
        if err != McError.NoError:
            return False, None

        position = [self.POS_NC] * self.__AXIS_MAX
        for i in range(min(len(pos), len(position))):
            if (pos[i] == MotionControl.PositionNotControl):
                continue
            position[i] = float(System.Convert.ToDouble(pos[i]))
#___     print(type(position), type(position[0]))
#___     for i in position:
#___         if (i == self.POS_NC):
#___             print("-")
#___         else:
#___             print(i)
        return (err == McError.NoError), position

    def WriteFlashMemory(self):
        """フラッシュメモリ書き込み (書き込みが終わるまで制御を返しません。書き込みは10秒程度かかります。)
        戻り値
            err		実行結果: true=成功, false=失敗
        """
        err = self.__mc.WriteFlashMemory()
        return (err == McError.NoError)


